import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { Owner } from '../../models/owner.model';
import { Structure } from '../../models/structure.model';
import { StructureWithOwners } from '../../models/structureWithOwners.model';
import { TempUser } from '../../models/temp-user.model';
import { User } from '../../models/user.model';
import { NotificationService } from '../../services/notification.service';
import { StructureService } from '../../services/structure.service';
import { ButtonType } from '../../shared/components/button/buttonType.enum';
import { Utils } from '../../utils/utils';
import { ProfileService } from '../services/profile.service';

@Component({
  selector: 'app-structure-members-management',
  templateUrl: './structure-members-management.component.html',
  styleUrls: ['./structure-members-management.component.scss'],
})
export class StructureMembersManagementComponent implements OnInit {
  public structure: Structure;
  public structureWithOwners: StructureWithOwners;
  public tempUsers: TempUser[];
  public tempUserToCancel: TempUser;
  public memberToExclude: Owner;
  public addMemberModalOpenned: boolean = false;
  public excludeModalOpenned = false;
  public cancelAddTempUserModalOpenned = false;
  public buttonTypeEnum = ButtonType;

  constructor(
    private route: ActivatedRoute,
    private structureService: StructureService,
    private profileService: ProfileService,
    private notificationService: NotificationService
  ) {}

  ngOnInit(): void {
    this.route.data.subscribe(async (data) => {
      if (data.structure) {
        this.structure = new Structure(data.structure);
        let currentProfile = await this.profileService.getProfile();
        this.structureService.getStructureWithOwners(data.structure._id, currentProfile).subscribe((s) => {
          this.structureWithOwners = s;
        });
        this.structureService.getTempUsers(data.structure._id).subscribe((data) => {
          this.tempUsers = data;
        });
      }
    });
  }
  public goBack(): void {
    history.back();
  }
  public displayJobEmployer(profile: User): string {
    return new Utils().getJobEmployer(profile);
  }
  public displayMemberName(member: Owner): string {
    return member.name + ' ' + member.surname.toUpperCase();
  }

  public excludeMember(member: Owner, shouldExclude: boolean): void {
    this.excludeModalOpenned = false;
    if (shouldExclude) {
      this.structureService.removeOwnerFromStructure(member._id, this.structure._id).subscribe(
        () => {
          this.structureWithOwners.owners = this.structureWithOwners.owners.filter((obj) => obj._id !== member._id);
          this.notificationService.showSuccess(
            `${this.displayMemberName(member)} a bien été exclu de ${this.structure.structureName}`,
            ''
          );
        },
        () => {
          this.notificationService.showError(
            `${this.displayMemberName(member)} n'a pas pu être exclu de ${
              this.structure.structureName
            }. Merci de réessayer plus tard.`,
            "Echec de l'exclusion"
          );
        }
      );
    }
  }

  public cancelAddTempUser(member: TempUser, shouldExclude: boolean): void {
    this.cancelAddTempUserModalOpenned = false;
    if (shouldExclude) {
      this.structureService.removeTempUserFromStructure(member._id, this.structure._id).subscribe(
        () => {
          this.tempUsers = this.tempUsers.filter((obj) => obj._id !== member._id);
          this.notificationService.showSuccess(
            `La demande d'ajout de ${member.email} à ${this.structure.structureName} a bien été annulée`,
            ''
          );
        },
        () => {
          this.notificationService.showError(
            `La demande d'ajout de ${member.email} à ${this.structure.structureName} n'a pas pu être annulée. Merci de réessayer plus tard.`,
            "Echec de l'annulation"
          );
        }
      );
    }
  }

  public closeAddMemberModal(memberAddRequested: boolean): void {
    this.addMemberModalOpenned = false;
    if (memberAddRequested) {
      this.ngOnInit();
      this.notificationService.showSuccess(`La demande d'ajout a bien été effectuée`, '');
    }
  }
}
