import { Injectable } from '@angular/core';
import { UntypedFormGroup } from '@angular/forms';
import { Structure } from '../models/structure.model';
import { User } from '../models/user.model';
import { Demarches } from '../shared/enum/demarches.enum';
import { FreeWorkshop } from '../shared/enum/freeWorkshop.enum';
import { OtherServices } from '../shared/enum/otherServices.enum';
import { AccessModality } from '../structure-list/enum/access-modality.enum';
import { Equipment } from '../structure-list/enum/equipment.enum';
import { PublicCategorie } from '../structure-list/enum/public.enum';

@Injectable({
  providedIn: 'root',
})
export class Utils {
  public modifyPhoneInput(form: UntypedFormGroup, controlName: string, phoneNumber: string): void {
    // Take length of phone number without spaces.
    const phoneNoSpace = phoneNumber.replace(/\s/g, '');
    // Check to refresh every 2 number.
    if (phoneNoSpace.length % 2 === 0) {
      // Add space every 2 number
      form.get(controlName).setValue(phoneNoSpace.replace(/(?!^)(?=(?:\d{2})+$)/g, ' ')); //NOSONAR
    }
  }

  public modifyPhoneValue(phoneNumber: string): string {
    // Take length of phone number without spaces.
    const phoneNoSpace = phoneNumber.replace(/\s/g, '');
    // Check to refresh every 2 number.
    if (phoneNoSpace.length % 2 === 0) {
      // Add space every 2 number
      return phoneNoSpace.replace(/(?!^)(?=(?:\d{2})+$)/g, ' '); //NOSONAR
    }
    return phoneNumber;
  }

  public getJobEmployer(userProfile: User): string {
    const jobName = userProfile.job?.name;
    const employerName = userProfile.employer?.name;
    if (!jobName && !employerName) return '';
    if (!jobName && employerName) return employerName;
    if (jobName && !employerName) return jobName;
    if (jobName && employerName) return `${jobName}, ${employerName}`;
  }

  public getAccessLabel(accessModality: AccessModality): string {
    switch (accessModality) {
      case AccessModality.free:
        return 'Accès libre';
      case AccessModality.meeting:
        return 'Sur rendez-vous';
      case AccessModality.meetingOnly:
        return 'Uniquement sur RDV';
      case AccessModality.numeric:
        return 'Téléphone / Visio';
      default:
        return null;
    }
  }

  public hasSocialNetwork(structure: Structure): boolean {
    return (
      (structure.facebook !== null && structure.facebook !== '') ||
      (structure.instagram !== null && structure.instagram !== '') ||
      (structure.linkedin !== null && structure.linkedin !== '') ||
      (structure.twitter !== null && structure.twitter !== '')
    );
  }

  public hasWebsite(structure: Structure): boolean {
    return structure.website !== null && structure.website !== '';
  }

  public getPublicLabel(tagetPublic: PublicCategorie): string {
    switch (tagetPublic) {
      case PublicCategorie.young:
        return 'Jeunes (16 - 25 ans)';
      case PublicCategorie.adult:
        return 'Adultes';
      case PublicCategorie.elderly:
        return 'Séniors (+ de 65 ans)';
      case PublicCategorie.all:
        return 'Tout public';
      case PublicCategorie.under16Years:
        return 'Moins de 16 ans';
      case PublicCategorie.women:
        return 'Uniquement femmes';
      default:
        return null;
    }
  }

  public getDigitalHelpLabel(digitalHelp: Demarches) {
    return Demarches[digitalHelp];
  }

  public getWorkshopPricingLabel(freeWorkshop: FreeWorkshop) {
    return FreeWorkshop[freeWorkshop];
  }

  public hasWifi(equipmentsAndServices: string[]) {
    return equipmentsAndServices.includes('wifiEnAccesLibre');
  }

  /**
   * Verify that a structure has equipments and checks that their value is > 0
   */
  public hasEquipments(
    equipmentsAndServices: string[],
    nbComputers: number,
    nbNumericTerminal: number,
    nbPrinters: number,
    nbScanners: number,
    nbTablets: number
  ): boolean {
    return equipmentsAndServices.length && nbComputers + nbPrinters + nbTablets + nbNumericTerminal + nbScanners > 0;
  }

  public filterOnlyEquipments(equipmentsAndServices: string[]): string[] {
    return equipmentsAndServices.filter((equipment) =>
      ['ordinateurs', 'tablettes', 'bornesNumeriques', 'imprimantes', 'scanners'].includes(equipment)
    );
  }

  public getEquipmentsLabelAndValue(equipment: Equipment, number: number): string {
    switch (equipment) {
      case Equipment.wifi:
        return 'Wifi en accès libre';
      case Equipment.bornes:
        return number > 1 ? `Bornes numériques : ${number}` : `Borne numérique : 1`;
      case Equipment.printer:
        return number > 1 ? `Imprimantes : ${number}` : `Imprimante : 1`;
      case Equipment.tablet:
        return number > 1 ? `Tablettes : ${number}` : `Tablette : 1`;
      case Equipment.computer:
        return number > 1 ? `Ordinateurs : ${number}` : `Ordinateur : 1`;
      case Equipment.scanner:
        return number > 1 ? `Scanners : ${number}` : `Scanner : 1`;
      default:
        return null;
    }
  }

  public filterOtherServices(equipmentsAndServices: string[]): string[] {
    return equipmentsAndServices.filter((equipment) =>
      [
        'reconditionnementsDeMateriel',
        'accesLivresInformatiques',
        'venteMaterielPrixSolidaire',
        'donDeMateriels',
      ].includes(equipment)
    );
  }

  public getOtherServicesLabel(otherService: OtherServices) {
    return OtherServices[otherService];
  }
}
